@@ -44,6 +44,8 @@
 /* NGHTTP3_MIN_RBLEN is the minimum length of nghttp3_ringbuf */
 #define NGHTTP3_MIN_RBLEN 4
 
+nghttp3_objalloc_def(stream, nghttp3_stream, oplent);
+
 int nghttp3_stream_new(nghttp3_stream **pstream, int64_t stream_id,
                        const nghttp3_stream_callbacks *callbacks,
                        nghttp3_objalloc *out_chunk_objalloc,
@@ -60,7 +62,7 @@ int nghttp3_stream_new(nghttp3_stream **pstream, int64_t stream_id,
   stream->out_chunk_objalloc = out_chunk_objalloc;
   stream->stream_objalloc = stream_objalloc;
 
-  nghttp3_tnode_init(&stream->node, stream_id, NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&stream->node, stream_id);
 
   nghttp3_ringbuf_init(&stream->frq, 0, sizeof(nghttp3_frame_entry), mem);
   nghttp3_ringbuf_init(&stream->chunks, 0, sizeof(nghttp3_buf), mem);
@@ -74,7 +76,7 @@ int nghttp3_stream_new(nghttp3_stream **pstream, int64_t stream_id,
   stream->tx.offset = 0;
   stream->rx.http.status_code = -1;
   stream->rx.http.content_length = -1;
-  stream->rx.http.pri = NGHTTP3_DEFAULT_URGENCY;
+  stream->rx.http.pri.urgency = NGHTTP3_DEFAULT_URGENCY;
   stream->error_code = NGHTTP3_H3_NO_ERROR;
 
   if (callbacks) {
@@ -143,6 +145,9 @@ static void delete_frq(nghttp3_ringbuf *frq, const nghttp3_mem *mem) {
     case NGHTTP3_FRAME_HEADERS:
       nghttp3_frame_headers_free(&frent->fr.headers, mem);
       break;
+    case NGHTTP3_FRAME_PRIORITY_UPDATE:
+      nghttp3_frame_priority_update_free(&frent->fr.priority_update, mem);
+      break;
     default:
       break;
     }
@@ -245,7 +250,7 @@ int nghttp3_stream_fill_outq(nghttp3_stream *stream) {
   int data_eof;
   int rv;
 
-  for (; nghttp3_ringbuf_len(frq) && !nghttp3_stream_outq_is_full(stream) &&
+  for (; nghttp3_ringbuf_len(frq) &&
          stream->unsent_bytes < NGHTTP3_MIN_UNSENT_BYTES;) {
     frent = nghttp3_ringbuf_get(frq, 0);
 
@@ -286,6 +291,8 @@ int nghttp3_stream_fill_outq(nghttp3_stream *stream) {
       if (rv != 0) {
         return rv;
       }
+      nghttp3_frame_priority_update_free(&frent->fr.priority_update,
+                                         stream->mem);
       break;
     default:
       /* TODO Not implemented */
@@ -348,10 +355,18 @@ int nghttp3_stream_write_settings(nghttp3_stream *stream,
   iv[2].id = NGHTTP3_SETTINGS_ID_QPACK_BLOCKED_STREAMS;
   iv[2].value = local_settings->qpack_blocked_streams;
 
+  if (local_settings->h3_datagram) {
+    iv[fr.settings.niv].id = NGHTTP3_SETTINGS_ID_H3_DATAGRAM;
+    iv[fr.settings.niv].value = 1;
+
+    ++fr.settings.niv;
+  }
+
   if (local_settings->enable_connect_protocol) {
+    iv[fr.settings.niv].id = NGHTTP3_SETTINGS_ID_ENABLE_CONNECT_PROTOCOL;
+    iv[fr.settings.niv].value = 1;
+
     ++fr.settings.niv;
-    iv[3].id = NGHTTP3_SETTINGS_ID_ENABLE_CONNECT_PROTOCOL;
-    iv[3].value = 1;
   }
 
   len = nghttp3_frame_write_settings_len(&fr.settings.hd.length, &fr.settings);
@@ -688,11 +703,6 @@ int nghttp3_stream_write_qpack_decoder_stream(nghttp3_stream *stream) {
   return nghttp3_stream_outq_add(stream, &tbuf);
 }
 
-int nghttp3_stream_outq_is_full(nghttp3_stream *stream) {
-  /* TODO Verify that the limit is reasonable. */
-  return nghttp3_ringbuf_len(&stream->outq) >= 1024;
-}
-
 int nghttp3_stream_outq_add(nghttp3_stream *stream,
                             const nghttp3_typed_buf *tbuf) {
   nghttp3_ringbuf *outq = &stream->outq;
@@ -806,8 +816,8 @@ int nghttp3_stream_require_schedule(nghttp3_stream *stream) {
           !(stream->flags & NGHTTP3_STREAM_FLAG_READ_DATA_BLOCKED));
 }
 
-nghttp3_ssize nghttp3_stream_writev(nghttp3_stream *stream, int *pfin,
-                                    nghttp3_vec *vec, size_t veccnt) {
+size_t nghttp3_stream_writev(nghttp3_stream *stream, int *pfin,
+                             nghttp3_vec *vec, size_t veccnt) {
   nghttp3_ringbuf *outq = &stream->outq;
   size_t len = nghttp3_ringbuf_len(outq);
   size_t i = stream->outq_idx;
@@ -846,10 +856,10 @@ nghttp3_ssize nghttp3_stream_writev(nghttp3_stream *stream, int *pfin,
   *pfin = nghttp3_ringbuf_len(&stream->frq) == 0 && i == len &&
           (stream->flags & NGHTTP3_STREAM_FLAG_WRITE_END_STREAM);
 
-  return vec - vbegin;
+  return (size_t)(vec - vbegin);
 }
 
-int nghttp3_stream_add_outq_offset(nghttp3_stream *stream, size_t n) {
+void nghttp3_stream_add_outq_offset(nghttp3_stream *stream, size_t n) {
   nghttp3_ringbuf *outq = &stream->outq;
   size_t i;
   size_t len = nghttp3_ringbuf_len(outq);
@@ -873,8 +883,6 @@ int nghttp3_stream_add_outq_offset(nghttp3_stream *stream, size_t n) {
   stream->unsent_bytes -= n;
   stream->outq_idx = i;
   stream->outq_offset = offset;
-
-  return 0;
 }
 
 int nghttp3_stream_outq_write_done(nghttp3_stream *stream) {
@@ -884,8 +892,8 @@ int nghttp3_stream_outq_write_done(nghttp3_stream *stream) {
   return len == 0 || stream->outq_idx >= len;
 }
 
-static int stream_pop_outq_entry(nghttp3_stream *stream,
-                                 nghttp3_typed_buf *tbuf) {
+static void stream_pop_outq_entry(nghttp3_stream *stream,
+                                  nghttp3_typed_buf *tbuf) {
   nghttp3_ringbuf *chunks = &stream->chunks;
   nghttp3_buf *chunk;
 
@@ -918,8 +926,6 @@ static int stream_pop_outq_entry(nghttp3_stream *stream,
   };
 
   nghttp3_ringbuf_pop_front(&stream->outq);
-
-  return 0;
 }
 
 int nghttp3_stream_add_ack_offset(nghttp3_stream *stream, uint64_t n) {
@@ -948,10 +954,7 @@ int nghttp3_stream_add_ack_offset(nghttp3_stream *stream, uint64_t n) {
     }
 
     if (offset >= buflen) {
-      rv = stream_pop_outq_entry(stream, tbuf);
-      if (rv != 0) {
-        return rv;
-      }
+      stream_pop_outq_entry(stream, tbuf);
 
       offset -= buflen;
       ++npopped;
@@ -976,6 +979,7 @@ int nghttp3_stream_add_ack_offset(nghttp3_stream *stream, uint64_t n) {
   }
 
   stream->ack_offset = offset;
+  stream->ack_total += n;
 
   return 0;
 }
