@@ -24,12 +24,21 @@
  */
 #include "nghttp3_tnode_test.h"
 
-#include <CUnit/CUnit.h>
+#include <stdio.h>
 
 #include "nghttp3_tnode.h"
 #include "nghttp3_macro.h"
 #include "nghttp3_test_helper.h"
 
+static const MunitTest tests[] = {
+    munit_void_test(test_nghttp3_tnode_schedule),
+    munit_test_end(),
+};
+
+const MunitSuite tnode_suite = {
+    "/tnode", tests, NULL, 1, MUNIT_SUITE_OPTION_NONE,
+};
+
 static int cycle_less(const nghttp3_pq_entry *lhsx,
                       const nghttp3_pq_entry *rhsx) {
   const nghttp3_tnode *lhs = nghttp3_struct_of(lhsx, nghttp3_tnode, pe);
@@ -51,114 +60,116 @@ void test_nghttp3_tnode_schedule(void) {
   nghttp3_tnode *p;
 
   /* Schedule node with incremental enabled */
-  nghttp3_tnode_init(&node, 0, (1 << 7) | NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&node, 0);
+  node.pri.inc = 1;
 
   nghttp3_pq_init(&pq, cycle_less, mem);
 
   rv = nghttp3_tnode_schedule(&node, &pq, 0);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(0 == node.cycle);
+  assert_int(0, ==, rv);
+  assert_uint64(0, ==, node.cycle);
 
   /* Schedule another node */
-  nghttp3_tnode_init(&node2, 1, (1 << 7) | NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&node2, 1);
+  node.pri.inc = 1;
 
   rv = nghttp3_tnode_schedule(&node2, &pq, 0);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
   /* Rescheduling node with nwrite > 0 */
 
   rv = nghttp3_tnode_schedule(&node, &pq, 1000);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(1 == node.cycle);
+  assert_int(0, ==, rv);
+  assert_uint64(1, ==, node.cycle);
 
   /* Rescheduling node with nwrite == 0 */
 
   rv = nghttp3_tnode_schedule(&node, &pq, 0);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(1 == node.cycle);
+  assert_int(0, ==, rv);
+  assert_uint64(1, ==, node.cycle);
 
   nghttp3_pq_free(&pq);
 
   /* Schedule node without incremental */
-  nghttp3_tnode_init(&node, 0, NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&node, 0);
 
   nghttp3_pq_init(&pq, cycle_less, mem);
 
   rv = nghttp3_tnode_schedule(&node, &pq, 0);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(0 == node.cycle);
+  assert_int(0, ==, rv);
+  assert_uint64(0, ==, node.cycle);
 
   /* Schedule another node */
-  nghttp3_tnode_init(&node2, 1, NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&node2, 1);
 
   rv = nghttp3_tnode_schedule(&node2, &pq, 0);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
   /* Rescheduling node with nwrite > 0 */
 
   rv = nghttp3_tnode_schedule(&node, &pq, 1000);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(0 == node.cycle);
+  assert_int(0, ==, rv);
+  assert_uint64(0, ==, node.cycle);
 
   /* Rescheduling node with nwrit == 0 */
 
   rv = nghttp3_tnode_schedule(&node, &pq, 0);
 
-  CU_ASSERT(0 == rv);
-  CU_ASSERT(0 == node.cycle);
+  assert_int(0, ==, rv);
+  assert_uint64(0, ==, node.cycle);
 
   nghttp3_pq_free(&pq);
 
   /* Stream with lower stream ID takes precedence */
   nghttp3_pq_init(&pq, cycle_less, mem);
 
-  nghttp3_tnode_init(&node2, 1, NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&node2, 1);
 
   rv = nghttp3_tnode_schedule(&node2, &pq, 0);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
-  nghttp3_tnode_init(&node, 0, NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&node, 0);
 
   rv = nghttp3_tnode_schedule(&node, &pq, 0);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
   ent = nghttp3_pq_top(&pq);
 
   p = nghttp3_struct_of(ent, nghttp3_tnode, pe);
 
-  CU_ASSERT(0 == p->id);
+  assert_int64(0, ==, p->id);
 
   nghttp3_pq_free(&pq);
 
   /* Check the same reversing push order */
   nghttp3_pq_init(&pq, cycle_less, mem);
 
-  nghttp3_tnode_init(&node, 0, NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&node, 0);
 
   rv = nghttp3_tnode_schedule(&node, &pq, 0);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
-  nghttp3_tnode_init(&node2, 1, NGHTTP3_DEFAULT_URGENCY);
+  nghttp3_tnode_init(&node2, 1);
 
   rv = nghttp3_tnode_schedule(&node2, &pq, 0);
 
-  CU_ASSERT(0 == rv);
+  assert_int(0, ==, rv);
 
   ent = nghttp3_pq_top(&pq);
 
   p = nghttp3_struct_of(ent, nghttp3_tnode, pe);
 
-  CU_ASSERT(0 == p->id);
+  assert_int64(0, ==, p->id);
 
   nghttp3_pq_free(&pq);
 }
